<?php
/**
 * @file clase_zen_modelo_datos.php
 * @author Juan Belon
 * @access public
 * @copyright LGPL, GPL
 * @package zenphp
 * @version 1.0
 * @uses zenphp FrameWork
 * 
 * @brief Clase que genera un modelo de datos a partir de una lista de campos y de tablas
//TODO:por venir : scaffolding
 */
// +----------------------------------------------------------------------
// | zenphp.es
// +----------------------------------------------------------------------
if (!defined('ZF_SEGURO_ZEN')) exit('Acceso directo no permitido');
class zen_modelo_datos {
	/**
	 * Aplicacion web
	 *
	 * @var zen
	 */
	var $padre;
	/**
	 * Listado de campos de las tablas separados por comas, poner el identificador principal el primero
	 *
	 * @var str
	 */
	var $campos;
	/**
	 * Nombre de las tablas asociadas al modelo de datos
	 *
	 * @var str
	 */
	var $tablas;
	/**
	 * Condiciones para filtrar los datos en los listados
	 *
	 * @var str
	 */
	var $condiciones_where;
	/**
	 * Clase de BD MySQL
	 *
	 * @var zen_basedatos
	 */
	var $bd;
	/**
	 * Clase para manejar productos de categorias si la hubiera
	 *
	 * @var zen_modelo_datos
	 */
	var $subcategoria = null;
	/**
	 * clase html que usa el modelo
	 *
	 * @var zen_html_modelo_datos
	 */
	var $html;
	/**
	 * Modelo de datos para mostrar en la web
	 *
	 * @param zen $_padre
	 * @param str $campos
	 * @param str $tablas
	 * @param str $subcategoria
	 * @param str $condiciones_where
	 * @return zen_modelo_datos
	 */
	function zen_modelo_datos(&$_padre,$campos="",$tablas="",$subcategoria=null,$condiciones_where=""){
		$this->padre  =& $_padre;
		$this->bd     =  $this->padre->bd;
		$this->campos =  $campos;
		$this->tablas =  $tablas;
		$this->condiciones_where = $condiciones_where;
		$this->html   =  new zen_html_modelo_datos($this);
		if ($subcategoria!=''){
		 if (zen___carga_clase($subcategoria))
		  eval('$this->subcategoria = new '.$subcategoria.'($this)');
		}
	}
	 /**
	  * Devuelve una lista de categorias con las opciones pasadas...son todas opcionales
	  *
	  * @param str $where
	  * @param str $tablas
	  * @param str $from
	  * @param str $order
	  * @param str $limit
	  * @return array
	  */
	function obtener($campos="",$tablas="",$where="",$order="",$limit="",$num_campos=0){
		if (empty($campos)) $campos = $this->campos;		
		if (empty($tablas)) $tablas = $this->tablas;
		if (empty($where))  $where  = $this->condiciones_where;
		$r = $this->bd->seleccion($campos." from ".$tablas." ".$where." ".$order." ".$limit);
		if (!$r) return false;
		$tuplas = array();
		while ($tupla = $this->bd->obtener_fila($r)) {
			array_push($tuplas,$tupla);
		}
		@mysql_free_result($r);
		return $tuplas;
	}
	/**
	 * Obtiene el primer dato del modelo de datos
	 *
	 * @param str $campos
	 * @param str $tablas
	 * @param str $where
	 * @param str $orden
	 * @return array
	 */
	function obtener_primero($campos="",$tablas="",$where="",$orden=""){
		$dato = $this->obtener($campos,$tablas,$where,$orden," LIMIT 1");
		return $dato[0];
	}
	/**
	 * Si necesitamos usar otra BD distinta o no crear la clase zen_modelo_datos podemos usar esta funcion
	 *
	 * @param zen_basedatos $bd
	 * @param str $campos
	 * @param str $tablas
	 * @param str $where
	 * @param str $order
	 * @param str $limit
	 * @return array
	 */
	function obtener_BD(&$bd,$campos,$tablas,$where,$order,$limit){
		if (is_int($limit)) $limit = "LIMIT ".$limit;
		$r = $bd->seleccion($campos." from ".$tablas." ".$where." ".$order." ".$limit);
		if (!$r) return false;
		$tuplas = array();
		while ($tupla = $bd->obtener_fila($r)) {
			array_push($tuplas,$tupla);
		}
		@mysql_free_result($r);
		return $tuplas;
	}
	/**
	 * Devuelve el entero de $id si corresponde con la BD
	 *
	 * @param int $id
	 * @return int
	 */
	function existe($id)
	{
		$c = split(",",$this->campos);
		return intval($this->bd->seleccion_unica($c[0]." from ".$this->tablas." where ".$c[0]."=".intval($id)));
	}
	/**
	 * Inserta datos desde el array pasado con los campos pasados y devuelve el nuevo indentificador pasado
	 *
	 * @param array $datos
	 * @return int
	 */
	function insertar(&$datos){
		$c = split(",",$this->campos);		
		$sql = $this->tablas."(".$this->campos.") values(";
		$n = sizeof($datos);
		for($i=0; $i<sizeof($c);$i++){
			if ($i==0) $sql .= intval($datos[$i]);
			else {
				//if ($i<=$n) $sql .=",";
				$sql .=",'".addslashes($datos[$c[$i]])."'";
			}
			
		}
		$sql .=");";

		return $this->bd->insertar($sql);
	}
	/**
	 * Actualiza los datos pasados en el array $datos y devuelve el numero de tuplas afectadas
	 *
	 * @param array $datos
	 * @return int
	 */
	function actualizar($datos){
		$c = split(",",$this->campos);
		$sql = $this->tablas." SET ";
		//El id se deja para el final,como condicion del where
		for ($i=1; $i<count($c); $i++){
			if ($i>1 /*&& ($i<sizeof($datos)-1)*/) $sql .= ",";
			$sql .= $c[$i]."='".addslashes($datos[$c[$i]])."'";

		}
		$sql .= " where ".$c[0]."=".intval($datos[$c[0]]);

		return $this->bd->actualizar($sql);
	}
	/**
	 * Borra una tupla con un id especifico y devuelve el numero de campos borrados como resultado
	 *
	 * @param int $id
	 * @return int
	 */
	function borrar($id){
		$c = split(",",$this->campos);
		return $this->bd->borrar($this->tablas." where ".$c[0]."='".addslashes($id)."';");
	}
}
/**
 * Clase HTML usada para el modelo de datos aqui arriba
 *
 */
class zen_html_modelo_datos {
	/**
	 * Clase modelo de datos asociada a la clase HTML
	 *
	 * @var zen_modelo_datos
	 */
	var $padre;
	/**
	 * Constructor
	 *
	 * @param zen_modelo_datos $_padre
	 * @return zen_html_modelo_datos
	 */
	function zen_html_modelo_datos(&$_padre){
		$this->padre =& $_padre;
		//Esta clase depende sobre todo de las plantillas
		if (!zen___carga_clase('zen_plantilla')) die('No se pudo cargar la clase plantillas de zenphp');
	}
	/**
	 * Utilizando una clase plantilla y un fichero HTML devuelve el contenido relleno con los datos.
	 * Si rellenamos $fichero_base ,se colocan los datos del listado dentro de la base ,sustituyendolo por la $marca_base
	 * Si alguna de $reemplazos_base o $reemplazos_datos tiene contenido, se reemplazan previamente en su correspondiente plantilla
	 *
	 * @param str $fichero
	 * @param str $fichero_base
	 * @param str $marca_base
	 * @param array|null $reemplazos_base
	 * @param arrat|null $reemplazos_datos
	 * @return str : html
	 */
	function listado($fichero,$fichero_base="",$marca_base="",$reemplazos_base=null,$reemplazos_datos=null){
		$p = new zen_plantilla($fichero);
		$datos = $this->padre->obtener();
		if (is_array($reemplazos_datos)) $p->pasarAplantilla($reemplazos_datos);

		$aux = $p->contenido;
		
		$html = "";
		for ($i=0; $i<sizeof($datos); $i++){
			$p->contenido = $aux;
			$p->pasarAplantilla($datos[$i]);
			$html .= $p->contenido;
		}
		if (!empty($fichero_base) && !empty($marca_base)){
			$p2= $p;
			$p2->cargar($fichero_base);
			if (is_array($reemplazos_base))	$p2->pasarAplantilla($reemplazos_base);
			$p2->reemplazar($marca_base , $html);
			return $p2->contenido;
		} else return $html;
	}
	/**
	 * Obtiene un registro de la BD con id de la lista de campos de las tablas y devuelve el HTML. Necesita la plantilla donde se reemplazan los datos...
	 * El $fichero_plantilla se carga para meterle los datos del modelo_datos y los $reemplazos son un array que se reemplaza previamente en la plantilla por si le falta algo extra al modelo_datos
	 *
	 * @param str $fichero_plantilla
	 * @param str $reemplazos 
	 * @return str :html
	 */
	function editar($fichero_plantilla,&$reemplazos){
		$p = new zen_plantilla();
		if (!$p->cargar($fichero_plantilla)) return false;
		$c = split(",",$this->padre->campos);
		$tupla = array();
		if (isset($_REQUEST[$c[0]]))
		{
			$tupla = $this->padre->obtener($this->padre->campos,""," where ".$c[0]."=".intval($_REQUEST[$c[0]]),""," limit 1");
			if (!$tupla) return "Sin datos";
		}
		//Primero los reemplazos de usuario
		$p->pasarAplantilla($reemplazos);
		//Despues los datos del modelo de datos:
		if (count($tupla)>0)
		 $p->pasarAplantilla($tupla[0]);
		return $p->contenido;
	}
	/**
	 * Muestra un registro en concreto usando una plantilla
	 *
	 * @param int $id
	 * @param str $fichero_plantilla
	 * @return str : html
	 */
	function mostrar($id,$fichero_plantilla,$reemplazos){
		if (!$this->padre->existe($id)) return "Sin datos";
		$p = new zen_plantilla();
		$campos = split(",",$this->padre->campos);		
		if (!is_array($reemplazos)) $reemplazos = array();
		return $p->contenido_reemplaza($fichero_plantilla, //Mete en la plantilla el contenido
		array_merge( //de los arrays de datos con el identificador y los reemplazos previos juntos
			$this->padre->obtener_primero("","","where ".$campos[0]."=".intval($id),"","limit 1"),
			$reemplazos
		));
	}
	
}
?>